'use strict';

var _ = require('lodash'),
		ContextEventEmitter = require('./ContextEventEmitter'),
		ContextQueryBuilder = require('./ContextQueryBuilder');

class Context extends ContextEventEmitter {
	constructor(name) {
		super();

		var query = this.createQuery();

		this.name = name;
		this.data = null;
		this.error = null;
		this.query = query;

		/**
		 * When the query get updated
		 * the context must emit an updated
		 * event from the inside
		 */
		query.on('update', context => {
			this.emit('update');
		});
	}

	getData() {
		return this.data;
	}

	setData(dataTransferObject) {
		if(_.isUndefined(dataTransferObject) || !_.isObject(dataTransferObject)) {
			this.setError(this.INVALID_RESPONSE);

			return this;
		}

		var page = dataTransferObject.meta;

		if(page) {
			var pagination = this.query.pagination();

			pagination.setState({
				totalItems: page.totalItems,
				currentPage: page.currentPage,
				itemsPerPage: page.itemsPerPage
			});
		}

		this.data = dataTransferObject.data || null;
		this.error = null;

		return this;
	}

	/**
	 * @name Context#createQuery
	 * @description
	 * It's a wrapper to:
	 * ```
	 * Context.createQuery().from(this.name);
	 * ```
	 */
	createQuery() {
		return Context.createQuery().from(this.name);
	}

	/**
	 * @name Context#pagination
	 * @description
	 * Retrieve the `QueryBuilder` pagination
	 */
	pagination() {
		return this.query.pagination();
	}

	reset() {
		this.query.reset();

		return this;
	}

	update() {
		this.triggerUpdate(this);

		return this;
	}

	triggerUpdate() {
		return this.emit('update');
	}

	setError(reason) {
	}

	/**
	 * @name Context#toJSON
	 * @description
	 * This method is only a wrapper to `this.query.toJSON()`
	 */
	toJSON() {
		return this.query.toJSON();
	}
}

Context.createQuery = function () {
	return new ContextQueryBuilder();
};

_.extend(Context.prototype, {
	INVALID_RESPONSE: 'INVALID_RESPONSE'
});

module.exports = Context;
