'use strict';

var _ = require('lodash'),
		EventEmitter = require('events');

class ContextEventEmitter extends EventEmitter {
	constructor() {
		super();

		this._queue = [];
		this._pauseEvents = false;
	}

	pause() {
		this._pauseEvents = true;

		return this;
	}

	resume() {
		this._pauseEvents = false;
		this.finish();

		return this;
	}

	isPaused() {
		return this._pauseEvents;
	}

	schedule() {
		var args = _.toArray(arguments);

		this._queue.push(args);

		return this;
	}

	finish() {
		this._queue.forEach(args => {
			this.emit.apply(this, args);
		});

		this._queue.splice(0, this._queue.length);

		return this;
	}

	emit() {
		if(this.isPaused()) {
			return this.schedule.apply(this, arguments);
		}

		return EventEmitter.prototype.emit.apply(this, arguments);
	}
}

module.exports = ContextEventEmitter;
