'use strict';

var QueryFilter = require('./QueryFilter'),
		EventEmitter = require('events'),
		QuerySorting = require('./QuerySorting'),
		QueryPagination = require('./QueryPagination');

class QueryBuilder extends EventEmitter {
	constructor(resourceName) {
		super();

		this._filter = new QueryFilter();
		this._sorting = new QuerySorting();
		this._pagination = new QueryPagination();

		this.resourceName = resourceName || null;
	}

	from (resourceName) {
		this.resourceName = resourceName;

		return this;
	}

	where () {
		this._filter.where.apply(this._filter, arguments);

		return this;
	}

	orderBy () {
		this._sorting.sort.apply(this._sorting, arguments);

		return this;
	}

	pagination () {
		return this._pagination;
	}

	limit (limit) {
		this.pagination().setState({
			itemsPerPage: limit
		});

		return this;
	}

	toJSON () {
		return {
			filters: this._filter.toJSON(),
			pagination: this._pagination.toJSON(),
			sorting: this._sorting.toJSON()
		};
	}

	reset () {
		this._filter.reset();
		this._sorting.reset();
		this._pagination.reset();

		return this;
	}
}

module.exports = QueryBuilder;
