'use strict';

var _ = require('lodash'),
		EventEmitter = require('events');

class QueryFilter extends EventEmitter {
	constructor() {
		super();
		this._filters = [];
		this.operatorsArray = _.keys(this.operators).map(key => {
			return this.operators[key];
		});
	}

	reset () {
		this._filters = [];
	}

	where (name, operator, value) {
		if(arguments.length === 2) {
			value = operator;
			operator = this.operators.EQ;
		}

		if(!this.hasOperator(operator)) {
			return this;
		}

		this.addFilter(name, operator, value);

		return this;
	}

	hasOperator (operator) {
		return this.operatorsArray.indexOf(operator) > -1;
	}

	remove (name) {
		if(!name) {
			return this;
		}

		var i,
				filters = this._filters;

		for(i = 0; i < filters.length; i++) {
			if(filters[i].name === name) {
				filters.splice(i, 1);
			}
		}

		return this;
	}

	addFilter () {
		var filter;
		var args = _.toArray(arguments);

		if(_.isArray(args[0])) {
			args = args[0];
		}

		if(_.isObject(args[0])) {
			filter = args[0];
		}

		if(_.isUndefined(filter)) {
			filter = {
				name: args[0],
				operator: args[1],
				value: args[2]
			};
		}

		if(!filter) {
			return this;
		}

		var hasDuplicated = this._filters.some(current => {
			return current.name === filter.name && current.operator === filter.operator;
		});

		// prevent duplicated filters
		if(hasDuplicated) {
			return this;
		}

		this._filters.push(filter);

		return this;
	}

	addFilters (filters) {
		if(!_.isArray(filters)) {
			throw new Error('Must be an array');
		}

		filters.forEach(this.addFilter, this);

		return this;
	}

	setState () {
		return this.addFilter.apply(this, arguments);
	}

	toJSON () {
		return this._filters.slice();
	}
}

QueryFilter.prototype.operators = {
	EQ: '=',
	LT: '<',
	LTE: '<=',
	GT: '>',
	GTE: '>=',
	IN: 'in',
	ST: '^',
	END: '$',
	LK: '~'
};

QueryFilter.create = function (filters) {
	var instance = new QueryFilter();
	instance.setState(filters);

	return instance;
};

module.exports = QueryFilter;
