var _ = require('lodash'),
    assert = require('assert'),
    Context = require('../src/context/Context'),
    Repository = require('../src/Repository'),
    QueryBuilder = require('../src/query/QueryBuilder'),
    DataProvider = require('./MockDataProvider'),
    RepositoryConfig = require('../src/RepositoryConfig');

describe('repository', function () {
	var config,
      repository;

	beforeEach(function () {
		config = new RepositoryConfig({
			name: 'User',
			dataProvider: new DataProvider([
        {name: 'user 1'},
        {name: 'Clementine'},
        {name: 'user 3'}
      ])
		});

		repository = new Repository(config);
	});

	it('should create a context', function () {
		var context = repository.createContext('user-list');

		context.setData({
			data: {}
		});

		assert.deepEqual(context.data, {});
	});

	it('should remove a context', function () {
		var context = repository.createContext('context-1');

		assert.ok(_.isUndefined(
      repository.removeContext(context.name).getContext(context.name)
    ));
	});

	it('should return a created context that was created before with the given name', function () {
		var context = repository.createContext('user-list-3');

		assert.equal(context instanceof Context, true);
	});

	it('should modify context data', function () {
		var context = repository.createContext('user-list-2');

		context.setData({
			data: {}
		});

		var data = context.getData();

		data.name = 'John Constantine';
		data.age = 33;

		assert.deepEqual(context.getData().name, 'John Constantine');
	});

	it('should retrieve a resource', function () {
		repository.findOne(1).then(function (user) {
			assert.equal(user.name, 'Clementine');
		});
	});

	it('should retrieve resources', function () {
		var qb = new QueryBuilder();
		qb.from(repository.name).limit(3);

		repository.find(qb).then(res => res.data).then(users => {
			assert.equal(users.length, 3);
			assert.equal(users[0].name, 'user 1');
		});
	});

	it('should remove resources', function() {
		var context = repository.createContext('user-target-1');
		context.update();

		var ids = _.map(context.data, user => {
			return user.id;
		});

		repository.remove(ids);
	});

	it('should remove resource', function () {
		repository.findOne(1).then(function (user) {
			return repository.removeOne(user.id);
		});
	});

  it('should save a resource', function () {
    repository.saveOne({
      name: 'Jessie Pinkman'
    });
  });

  it('should save resources', function () {
    repository.save([
      {name: 'foo'},
      {name: 'foo'}
    ]);
  });
});
